@@ -1,9 +1,9 @@
 package Color::RGB::Util;
 
 our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
-our $DATE = '2020-06-07'; # DATE
+our $DATE = '2020-06-08'; # DATE
 our $DIST = 'Color-RGB-Util'; # DIST
-our $VERSION = '0.600'; # VERSION
+our $VERSION = '0.601'; # VERSION
 
 use 5.010001;
 use strict;
@@ -46,6 +46,12 @@ sub _min {
     $_[0] < $_[1] ? $_[0] : $_[1];
 }
 
+sub _wrap_h {
+    my $h = shift;
+    $h %= 360 if abs($h) > 360;
+    $h >= 0 ? $h : 360+$h;
+}
+
 sub assign_rgb_color {
     require Digest::SHA;
 
@@ -87,9 +93,9 @@ sub mix_2_rgb_colors {
     $pct //= 0.5;
 
     my ($r1, $g1, $b1) =
-        $rgb1 =~ $re_rgb or die "Invalid rgb1 color, must be in 'ffffff' form";
+        $rgb1 =~ $re_rgb or die "Invalid rgb1 color '$rgb1', must be in 'ffffff' form";
     my ($r2, $g2, $b2) =
-        $rgb2 =~ $re_rgb or die "Invalid rgb2 color, must be in 'ffffff' form";
+        $rgb2 =~ $re_rgb or die "Invalid rgb2 color '$rgb2', must be in 'ffffff' form";
     for ($r1, $g1, $b1, $r2, $g2, $b2) { $_ = hex $_ }
 
     return sprintf("%02x%02x%02x",
@@ -130,10 +136,10 @@ sub rand_rgb_color {
 
     $rgb1 //= '000000';
     my ($r1, $g1, $b1) =
-        $rgb1 =~ $re_rgb or die "Invalid rgb1 color, must be in 'ffffff' form";
+        $rgb1 =~ $re_rgb or die "Invalid rgb1 color '$rgb1', must be in 'ffffff' form";
     $rgb2 //= 'ffffff';
     my ($r2, $g2, $b2) =
-        $rgb2 =~ $re_rgb or die "Invalid rgb2 color, must be in 'ffffff' form";
+        $rgb2 =~ $re_rgb or die "Invalid rgb2 color '$rgb2', must be in 'ffffff' form";
     for ($r1, $g1, $b1, $r2, $g2, $b2) { $_ = hex $_ }
 
     return sprintf("%02x%02x%02x",
@@ -182,7 +188,7 @@ sub reverse_rgb_color {
     my ($rgb) = @_;
 
     my ($r, $g, $b) =
-        $rgb =~ $re_rgb or die "Invalid rgb color, must be in 'ffffff' form";
+        $rgb =~ $re_rgb or die "Invalid rgb color '$rgb', must be in 'ffffff' form";
     for ($r, $g, $b) { $_ = hex $_ }
 
     return sprintf("%02x%02x%02x", 255-$r, 255-$g, 255-$b);
@@ -192,7 +198,7 @@ sub rgb2grayscale {
     my ($rgb) = @_;
 
     my ($r, $g, $b) =
-        $rgb =~ $re_rgb or die "Invalid rgb color, must be in 'ffffff' form";
+        $rgb =~ $re_rgb or die "Invalid rgb color '$rgb', must be in 'ffffff' form";
     for ($r, $g, $b) { $_ = hex $_ }
 
     # basically we just average the R, G, B
@@ -204,7 +210,7 @@ sub rgb2int {
     my $rgb = shift;
 
     # just to check
-    $rgb =~ $re_rgb or die "Invalid rgb color, must be in 'ffffff' form";
+    $rgb =~ $re_rgb or die "Invalid rgb color '$rgb', must be in 'ffffff' form";
 
     hex($rgb);
 }
@@ -213,7 +219,7 @@ sub rgb2sepia {
     my ($rgb) = @_;
 
     my ($r, $g, $b) =
-        $rgb =~ $re_rgb or die "Invalid rgb color, must be in 'ffffff' form";
+        $rgb =~ $re_rgb or die "Invalid rgb color '$rgb', must be in 'ffffff' form";
     for ($r, $g, $b) { $_ = hex $_ }
 
     # reference: http://www.techrepublic.com/blog/howdoi/how-do-i-convert-images-to-grayscale-and-sepia-tone-using-c/120
@@ -230,9 +236,9 @@ sub rgb_diff {
     $algo //= 'euclidean';
 
     my ($r1, $g1, $b1) =
-        $rgb1 =~ $re_rgb or die "Invalid rgb1 color, must be in 'ffffff' form";
+        $rgb1 =~ $re_rgb or die "Invalid rgb1 color '$rgb1', must be in 'ffffff' form";
     my ($r2, $g2, $b2) =
-        $rgb2 =~ $re_rgb or die "Invalid rgb2 color, must be in 'ffffff' form";
+        $rgb2 =~ $re_rgb or die "Invalid rgb2 color '$rgb2', must be in 'ffffff' form";
     for ($r1, $g1, $b1, $r2, $g2, $b2) { $_ = hex $_ }
 
     my $dr2 = ($r1-$r2)**2;
@@ -274,9 +280,9 @@ sub rgb_distance {
     my ($rgb1, $rgb2) = @_;
 
     my ($r1, $g1, $b1) =
-        $rgb1 =~ $re_rgb or die "Invalid rgb1 color, must be in 'ffffff' form";
+        $rgb1 =~ $re_rgb or die "Invalid rgb1 color '$rgb1', must be in 'ffffff' form";
     my ($r2, $g2, $b2) =
-        $rgb2 =~ $re_rgb or die "Invalid rgb2 color, must be in 'ffffff' form";
+        $rgb2 =~ $re_rgb or die "Invalid rgb2 color '$rgb2', must be in 'ffffff' form";
     for ($r1, $g1, $b1, $r2, $g2, $b2) { $_ = hex $_ }
 
     (($r1-$r2)**2 + ($g1-$g2)**2 + ($b1-$b2)**2)**0.5;
@@ -301,7 +307,7 @@ sub rgb_luminance {
     my ($rgb) = @_;
 
     my ($r, $g, $b) =
-        $rgb =~ $re_rgb or die "Invalid rgb color, must be in 'ffffff' form";
+        $rgb =~ $re_rgb or die "Invalid rgb color '$rgb', must be in 'ffffff' form";
     for ($r, $g, $b) { $_ = hex $_ }
 
     return _rgb_luminance($r, $g, $b);
@@ -313,9 +319,9 @@ sub tint_rgb_color {
     $pct //= 0.5;
 
     my ($r1, $g1, $b1) =
-        $rgb1 =~ $re_rgb or die "Invalid rgb1 color, must be in 'ffffff' form";
+        $rgb1 =~ $re_rgb or die "Invalid rgb1 color '$rgb1', must be in 'ffffff' form";
     my ($r2, $g2, $b2) =
-        $rgb2 =~ $re_rgb or die "Invalid rgb2 color, must be in 'ffffff' form";
+        $rgb2 =~ $re_rgb or die "Invalid rgb2 color '$rgb2', must be in 'ffffff' form";
     for ($r1, $g1, $b1, $r2, $g2, $b2) { $_ = hex $_ }
 
     my $lum = _rgb_luminance($r1, $g1, $b1);
@@ -331,7 +337,7 @@ sub rgb2hsl {
     my ($rgb) = @_;
 
     my ($r, $g, $b) =
-        $rgb =~ $re_rgb or die "Invalid rgb color, must be in 'ffffff' form";
+        $rgb =~ $re_rgb or die "Invalid rgb color '$rgb', must be in 'ffffff' form";
     for ($r, $g, $b) { $_ = hex($_)/255 }
 
     my $max = $r;
@@ -382,7 +388,7 @@ sub rgb2hsv {
     my ($rgb) = @_;
 
     my ($r, $g, $b) =
-        $rgb =~ $re_rgb or die "Invalid rgb color, must be in 'ffffff' form";
+        $rgb =~ $re_rgb or die "Invalid rgb color '$rgb', must be in 'ffffff' form";
     for ($r, $g, $b) { $_ = hex($_)/255 }
 
     my $max = $r;
@@ -431,6 +437,7 @@ sub hsl2hsv {
     my $hsl = shift;
 
     my ($h, $s, $l) = split / /, $hsl;
+    $h>=0 && $h<=360 or $h = _wrap_h($h); $s>=0 && $s<=1 or die "Invalid S in HSL '$hsl', must be in 0-1"; $l>=0 && $l<=1 or die "Invalid L in HSL '$hsl', must be in 0-1";
     my $_h = $h;
     my $_s;
     my $_v;
@@ -447,6 +454,7 @@ sub hsv2hsl {
     my $hsv = shift;
 
     my ($h, $s, $v) = split / /, $hsv;
+    $h>=0 && $h<=360 or $h = _wrap_h($h); $s>=0 && $s<=1 or die "Invalid S in HSV '$hsv', must be in 0-1"; $v>=0 && $v<=1 or die "Invalid V in HSV '$hsv', must be in 0-1";
     my $_h = $h;
     my $_s = $s * $v;
     my $_l = (2-$s) * $v;
@@ -465,6 +473,7 @@ sub hsv2rgb {
     my $hsv = shift;
 
     my ($h, $s, $v) = split / /, $hsv;
+    $h>=0 && $h<=360 or $h = _wrap_h($h); $s>=0 && $s<=1 or die "Invalid S in HSV '$hsv', must be in 0-1"; $v>=0 && $v<=1 or die "Invalid V in HSV '$hsv', must be in 0-1";
 
     my $i = int($h/60);
     my $f = $h/60 - $i;
@@ -505,7 +514,7 @@ Color::RGB::Util - Utilities related to RGB colors
 
 =head1 VERSION
 
-This document describes version 0.600 of Color::RGB::Util (from Perl distribution Color-RGB-Util), released on 2020-06-07.
+This document describes version 0.601 of Color::RGB::Util (from Perl distribution Color-RGB-Util), released on 2020-06-08.
 
 =head1 SYNOPSIS
 
